@@ -37,8 +37,6 @@ Author: Ralph Mor, X Consortium
 static XtransConnInfo ConnectToPeer(char *networkIdsList,
 				    char **actualConnectionRet);
 
-#define Strstr strstr
-
 IceConn
 IceOpenConnection (
 	char 	   *networkIdsList,
@@ -68,8 +66,11 @@ IceOpenConnection (
 
     if (networkIdsList == NULL || *networkIdsList == '\0')
     {
-	strncpy (errorStringRet,
-	    "networkIdsList argument is NULL", errorLength);
+	if (errorStringRet && errorLength > 0) {
+	    strncpy (errorStringRet,
+		"networkIdsList argument is NULL", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 	return (NULL);
     }
 
@@ -93,7 +94,7 @@ IceOpenConnection (
     for (i = 0; i < _IceConnectionCount; i++)
     {
 	char *strptr;
-	if ((strptr = (char *) Strstr (
+	if ((strptr = (char *) strstr (
 	    networkIdsList, _IceConnectionStrings[i])) != NULL)
 	{
 	    char ch = *(strptr + strlen (_IceConnectionStrings[i]));
@@ -144,9 +145,12 @@ IceOpenConnection (
 	}
     }
 
-    if ((iceConn = (IceConn) malloc (sizeof (struct _IceConn))) == NULL)
+    if ((iceConn = malloc (sizeof (struct _IceConn))) == NULL)
     {
-	strncpy (errorStringRet, "Can't malloc", errorLength);
+	if (errorStringRet && errorLength > 0) {
+	    strncpy (errorStringRet, "Can't malloc", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 	return (NULL);
     }
 
@@ -158,8 +162,11 @@ IceOpenConnection (
     if ((iceConn->trans_conn = ConnectToPeer (networkIdsList,
 	&iceConn->connection_string)) == NULL)
     {
-	free ((char *) iceConn);
-	strncpy (errorStringRet, "Could not open network socket", errorLength);
+	free (iceConn);
+	if (errorStringRet && errorLength > 0) {
+	    strncpy (errorStringRet, "Could not open network socket", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 	return (NULL);
     }
 
@@ -194,21 +201,25 @@ IceOpenConnection (
     iceConn->connect_to_me = NULL;
     iceConn->protosetup_to_me = NULL;
 
-    if ((iceConn->inbuf = iceConn->inbufptr =
-	(char *) malloc (ICE_INBUFSIZE)) == NULL)
+    if ((iceConn->inbuf = iceConn->inbufptr = malloc (ICE_INBUFSIZE)) == NULL)
     {
 	_IceFreeConnection (iceConn);
-	strncpy (errorStringRet, "Can't malloc", errorLength);
+	if (errorStringRet && errorLength > 0) {
+	    strncpy (errorStringRet, "Can't malloc", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 	return (NULL);
     }
 
     iceConn->inbufmax = iceConn->inbuf + ICE_INBUFSIZE;
 
-    if ((iceConn->outbuf = iceConn->outbufptr =
-	(char *) calloc (1, ICE_OUTBUFSIZE)) == NULL)
+    if ((iceConn->outbuf = iceConn->outbufptr = calloc (1, ICE_OUTBUFSIZE)) == NULL)
     {
 	_IceFreeConnection (iceConn);
-	strncpy (errorStringRet, "Can't malloc", errorLength);
+	if (errorStringRet && errorLength > 0) {
+	    strncpy (errorStringRet, "Can't malloc", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 	return (NULL);
     }
 
@@ -224,8 +235,16 @@ IceOpenConnection (
     iceConn->saved_reply_waits = NULL;
     iceConn->ping_waits = NULL;
 
-    iceConn->connect_to_you = (_IceConnectToYouInfo *) malloc (
-	sizeof (_IceConnectToYouInfo));
+    iceConn->connect_to_you = malloc (sizeof (_IceConnectToYouInfo));
+    if (iceConn->connect_to_you == NULL)
+    {
+	_IceFreeConnection (iceConn);
+	if (errorStringRet && errorLength > 0) {
+	    strncpy (errorStringRet, "Can't malloc", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
+	return (NULL);
+    }
     iceConn->connect_to_you->auth_active = 0;
 
     /*
@@ -262,8 +281,11 @@ IceOpenConnection (
     if (ioErrorOccured)
     {
 	_IceFreeConnection (iceConn);
-	strncpy (errorStringRet, "IO error occured opening connection",
-	     errorLength);
+	if (errorStringRet && errorLength > 0) {
+	    strncpy (errorStringRet, "IO error occured opening connection",
+		 errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 	return (NULL);
     }
 
@@ -274,9 +296,12 @@ IceOpenConnection (
 	 */
 
 	_IceFreeConnection (iceConn);
-	strncpy (errorStringRet,
-	    "Internal error - did not receive the expected ByteOrder message",
-	     errorLength);
+	if (errorStringRet && errorLength > 0) {
+	    strncpy (errorStringRet,
+		"Internal error - did not receive the expected ByteOrder "
+		"message", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 	return (NULL);
     }
 
@@ -360,8 +385,11 @@ IceOpenConnection (
 
 	if (ioErrorOccured)
 	{
-	    strncpy (errorStringRet, "IO error occured opening connection",
-		errorLength);
+	    if (errorStringRet && errorLength > 0) {
+		strncpy (errorStringRet, "IO error occured opening connection",
+		    errorLength);
+		errorStringRet[errorLength - 1] = '\0';
+	    }
 	    _IceFreeConnection (iceConn);
 	    iceConn = NULL;
 	}
@@ -371,9 +399,12 @@ IceOpenConnection (
 	    {
 		if (reply.connection_reply.version_index >= _IceVersionCount)
 		{
-		    strncpy (errorStringRet,
-	    		"Got a bad version index in the Connection Reply",
-			errorLength);
+		    if (errorStringRet && errorLength > 0) {
+			strncpy (errorStringRet,
+			    "Got a bad version index in the Connection Reply",
+			    errorLength);
+			errorStringRet[errorLength - 1] = '\0';
+		    }
 
 		    free (reply.connection_reply.vendor);
 		    free (reply.connection_reply.release);
@@ -392,7 +423,7 @@ IceOpenConnection (
 			iceConn->connection_string;
 		    _IceConnectionCount++;
 
-		    free ((char *) iceConn->connect_to_you);
+		    free (iceConn->connect_to_you);
 		    iceConn->connect_to_you = NULL;
 
 		    iceConn->connection_status = IceConnectAccepted;
@@ -402,8 +433,11 @@ IceOpenConnection (
 	    {
 		/* Connection failed */
 
-		strncpy (errorStringRet, reply.connection_error.error_message,
-		    errorLength);
+		if (errorStringRet && errorLength > 0) {
+		    strncpy (errorStringRet,
+			reply.connection_error.error_message, errorLength);
+		    errorStringRet[errorLength - 1] = '\0';
+		}
 
 		free (reply.connection_error.error_message);
 
@@ -426,7 +460,7 @@ IceOpenConnection (
 }
 
 
-
+
 IcePointer
 IceGetConnectionContext (
 	IceConn    iceConn
@@ -436,7 +470,7 @@ IceGetConnectionContext (
 }
 
 
-
+
 /* ------------------------------------------------------------------------- *
  *                            local routines                                 *
  * ------------------------------------------------------------------------- */
@@ -451,9 +485,9 @@ ConnectToPeer (char *networkIdsList, char **actualConnectionRet)
     char* address;
     char *ptr, *endptr, *delim;
     int  madeConnection = 0;
-    int  len, retry;
-    int  connect_stat;
-    int  address_size;
+    size_t  len;
+    int  retry, connect_stat;
+    size_t  address_size;
     XtransConnInfo trans_conn = NULL;
 
     *actualConnectionRet = NULL;
