@@ -35,7 +35,7 @@ in this Software without prior written authorization from The Open Group.
 #include <X11/Xtrans/Xtrans.h>
 #include <stdio.h>
 
-
+
 Status
 IceListenForWellKnownConnections (
 	char		*port,
@@ -58,18 +58,21 @@ IceListenForWellKnownConnections (
 	*listenObjsRet = NULL;
 	*countRet = 0;
 
-        strncpy (errorStringRet,
-	    "Cannot establish any listening sockets", errorLength);
+	if (errorStringRet && errorLength > 0) {
+            strncpy (errorStringRet,
+		"Cannot establish any listening sockets", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 
+	free (transConns);
 	return (0);
     }
 
-    if ((listenObjs = (struct _IceListenObj *) malloc (
-	transCount * sizeof (struct _IceListenObj))) == NULL)
+    if ((listenObjs = malloc (transCount * sizeof (struct _IceListenObj))) == NULL)
     {
 	for (i = 0; i < transCount; i++)
 	    _IceTransClose (transConns[i]);
-	free ((char *) transConns);
+	free (transConns);
 	return (0);
     }
 
@@ -92,19 +95,24 @@ IceListenForWellKnownConnections (
     {
 	*listenObjsRet = NULL;
 
-        strncpy (errorStringRet,
-	    "Cannot establish any listening sockets", errorLength);
+	if (errorStringRet && errorLength > 0) {
+            strncpy (errorStringRet,
+		"Cannot establish any listening sockets", errorLength);
+	    errorStringRet[errorLength - 1] = '\0';
+	}
 
 	status = 0;
     }
     else
     {
-	*listenObjsRet = (IceListenObj *) malloc (
-	    *countRet * sizeof (IceListenObj));
+	*listenObjsRet = malloc (*countRet * sizeof (IceListenObj));
 
 	if (*listenObjsRet == NULL)
 	{
-	    strncpy (errorStringRet, "Malloc failed", errorLength);
+	    if (errorStringRet && errorLength > 0) {
+		strncpy (errorStringRet, "Malloc failed", errorLength);
+		errorStringRet[errorLength - 1] = '\0';
+	    }
 
 	    status = 0;
 	}
@@ -112,17 +120,19 @@ IceListenForWellKnownConnections (
 	{
 	    for (i = 0; i < *countRet; i++)
 	    {
-		(*listenObjsRet)[i] = (IceListenObj) malloc (
-		    sizeof (struct _IceListenObj));
+		(*listenObjsRet)[i] = malloc (sizeof (struct _IceListenObj));
 
 		if ((*listenObjsRet)[i] == NULL)
 		{
-		    strncpy (errorStringRet, "Malloc failed", errorLength);
+		    if (errorStringRet && errorLength > 0) {
+		        strncpy (errorStringRet, "Malloc failed", errorLength);
+			errorStringRet[errorLength - 1] = '\0';
+		    }
 
 		    for (j = 0; j < i; j++)
-			free ((char *) (*listenObjsRet)[j]);
+			free ((*listenObjsRet)[j]);
 
-		    free ((char *) *listenObjsRet);
+		    free (*listenObjsRet);
 		    *listenObjsRet = NULL;
 
 		    status = 0;
@@ -152,8 +162,8 @@ IceListenForWellKnownConnections (
 	    _IceTransClose (transConns[i]);
     }
 
-    free ((char *) listenObjs);
-    free ((char *) transConns);
+    free (listenObjs);
+    free (transConns);
 
     return (status);
 }
